//-----------------------------------------------------------------------------
//
//   Filename : csco_ht_coverage.e
//   Author   : Lisa Huang
//   Language : 'e'
//
//
//   General Description: Impliments comprehensive functional coverage
//                        statistics for HT evc. Tried to impliment as
//                        much as specman currently allows. This could
//                        defintely be improved when the specman 4.0 release
//                        becomes available.
//
//
//   History:
//   07/20/01 : LH initial coding
//
//-----------------------------------------------------------------------------


// ***** BEGIN LICENSE BLOCK *****
// Version: MPL 1.1
// The contents of this file are subject to the Mozilla Public License Version
// 1.1 (the "License"); you may not use this file except in compliance with
// the License. You may obtain a copy of the License at
// http:                                                   // www.mozilla.org/MPL/ or ./mpl_1_1.txt
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the License.
// The Original Code is Cisco Hypertransport eVC.
// The Initial Developer of the Original Code is
// Brian Van Essen, Lisa Huang, and Sean Smith.
// Portions created by the Initial Developer are Copyright (C) 2002
// the Initial Developer. All Rights Reserved.
// Contributor(s): Brian Van Essen, Lisa Huang, and Sean Smith
// ***** END LICENSE BLOCK *****

<'
extend csco_ht_packet {
  // event to cover any pkts received
  cover rx_pkt_cov_event is {
    item command;
    item packet_length using illegal = ((packet_length < 4) &&
                                        (packet_length > 64));
    cross command, packet_length;
  };

  // event to cover any pkts transmited
  cover tx_pkt_cov_event is {
    item command;
    item packet_length using illegal = ((packet_length < 4) &&
                                        (packet_length > 64));

    item vChan;
    item passing_rule;

    cross command, packet_length;
    cross command, vChan, passing_rule using illegal =
    (((command == Nop) && (vChan != NA)) ||
     ((command == Nop) && (passing_rule != NA)) ||
     ((command == Flush) && (vChan != NonPosted)) ||
     ((command == Flush) && (passing_rule == NA)) ||
     ((command == Write) && (vChan == NA)) ||
     ((command == Write) && (vChan == Response)) ||
     ((command == Write) && (passing_rule == NA)) ||
     ((command == Read) && (vChan != NonPosted)) ||
     ((command == Read) && (passing_rule == NA)) ||
     ((command == RdResponse) && (vChan != Response)) ||
     ((command == RdResponse) && (passing_rule == NA)) ||
     ((command == TgtDone) && (vChan != Response)) ||
     ((command == TgtDone) && (passing_rule == NA)) ||
     ((command == Broadcast) && (vChan == NA)) ||
     ((command == Broadcast) && (vChan == Response)) ||
     ((command == Broadcast) && (passing_rule == NA)) ||
     ((command == Fence) && (vChan != Posted)) ||
     ((command == Fence) && (passing_rule == NA)) ||
     ((command == Sync) && (vChan != NA)) ||
     ((command == Sync) && (passing_rule != NA)) ||
     ((command == Reserved) && (vChan != NA)) ||
     ((command == Reserved) && (passing_rule != NA)) ||
     ((command == DoubleWord_Read) && (vChan != Response)) ||
     ((command == DoubleWord_Read) && (passing_rule != NA)) ||
     ((command == Byte_Read) && (vChan != Response)) ||
     ((command == Byte_Read) && (passing_rule != NA)) ||
     ((command == DoubleWord_Write) && (vChan == NA)) ||
     ((command == DoubleWord_Write) && (vChan == Response)) ||
     ((command == DoubleWord_Write) && (passing_rule != NA)) ||
     ((command == Byte_Write) && (vChan == NA)) ||
     ((command == Byte_Write) && (vChan == Response)) ||
     ((command == Byte_Write) && (passing_rule != NA)));
  };

  // event to cover transmitting read pkts
  cover rd_tx_pkt_cov_event is {
    item command       using illegal = (command != Read);
    item packet_length using illegal = (packet_length != 8);
  };

  when Read'command csco_ht_packet {
    cover rd_tx_pkt_cov_event is also {
      // split 40-bit address into region and address
      item region : uint (bits: 16) = Addr[39:24];
      item addr   : uint (bits: 24) = Addr[23:0];

      item access_type;

      item data_type;
      item Mask_Count;

      cross data_type, Mask_Count;
    };
  };

  // event to cover transmitting write pkts
  cover wr_tx_pkt_cov_event is {
    item command       using illegal = (command != Write);
    item packet_length using illegal = (packet_length != 8);
  };

  when Write'command csco_ht_packet {
    cover wr_tx_pkt_cov_event is also {
      // split 40-bit address into region and address
      item region : uint (bits: 16) = Addr[39:24];
      item addr   : uint (bits: 24) = Addr[23:0];

      item access_type;
      item Mask_Count;

      item data_type;

      // fields associated with interrrupts
      item interrupt;
      item MT;
      item TM;
      item DM;
      item IntrDest;
      item Vector;

      cross data_type, Mask_Count;
      cross interrupt, MT, TM, DM, IntrDest, Vector;
    };
  };

  // event to cover transmitting broadcast pkts
  cover bc_tx_pkt_cov_event is {
    item command       using illegal = (command != Broadcast);
    item packet_length using illegal = (packet_length != 8);
  };

  when Broadcast'command csco_ht_packet {
    cover bc_tx_pkt_cov_event is also {
      // fields associated with interrrupts
      item interrupt;
      item MT using illegal = (MT != EOI);
      item Vector;

      cross interrupt, MT, Vector;
    };
  };
};

extend csco_ht_channel_interface {
  // cover if a tx data pkt has every been interrupted
  cover intrp_tx_data_pkt is {
    item interrupted_tx_data_buffer_type using illegal =
    ((interrupted_tx_data_buffer_type != Data));
  };

  // cover if a tx ctl pkt has every been interrupted
  cover intrp_tx_ctl_pkt is {
    item interrupted_tx_ctl_buffer_type using illegal =
    ((interrupted_tx_ctl_buffer_type != Ctl));
  };
};

'>
