/////////////////////////////////////////////////////////////////////////////
//
//   Filename : csco_ht_coverage.e
//   Author   : Vida Vakilotojar
//   Language : 'e'
//
//
//   General Description: Impliments comprehensive functional coverage
//                        statistics for HT evc.
//
//
//   History:
//   01/16/02 : initial coding
//
////////////////////////////////////////////////////////////////////////////

// ***** BEGIN LICENSE BLOCK *****
// Version: MPL 1.1
// The contents of this file are subject to the Mozilla Public License Version
// 1.1 (the "License"); you may not use this file except in compliance with
// the License. You may obtain a copy of the License at
// http:                                                   // www.mozilla.org/MPL/ or ./mpl_1_1.txt
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the License.
// The Original Code is Cisco Hypertransport eVC.
// The Initial Developer of the Original Code is
// Brian Van Essen, Lisa Huang, and Sean Smith.
// Portions created by the Initial Developer are Copyright (C) 2002
// the Initial Developer. All Rights Reserved.
// Contributor(s): Brian Van Essen, Lisa Huang, and Sean Smith
// ***** END LICENSE BLOCK *****


<'
import csco_ht_declarations.e;

define max_buffer_counter_width 3;
define max_buffer_counter_size  5;

struct global_cover_t {
};

extend sys {
  global_cover: global_cover_t;
};

extend sys {
    setup() is also {
      set_config (cover, max_int_buckets,256);
    };
};

define <add_coverage_on_tx_port'statement> "add_coverage_on_tx_port <instance'name>" as {
  extend global_cover_t {
    <instance'name>_collision_addr: uint (bits: 35);
  };

  extend csco_ht_channel_interface {
    event <instance'name>_rr_cover;
    event <instance'name>_rw_cover;
    event <instance'name>_wr_cover;
    event <instance'name>_ww_cover;
    event <instance'name>_rrr_cover;
    event <instance'name>_rrw_cover;
    event <instance'name>_rwr_cover;
    event <instance'name>_rww_cover;
    event <instance'name>_wrr_cover;
    event <instance'name>_wrw_cover;
    event <instance'name>_wwr_cover;
    event <instance'name>_www_cover;

    // Coverage code for triplet read/write commands with cache line collision
    do_cover_rw_sequence (instance: channel_instance_e) is also {
      if instance == <instance'name> &&
            first_rw.command in [Write, Read] && second_rw.command in [Write, Read] {
        var first_rw_addr: uint (bits:35) = first_rw.addr;
        var collision: bool = FALSE;
        for i from 0 to 1 {
          var second_rw_addr: uint (bits:35) = second_rw.addr;
          for j from 0 to 1 {
            var third_rw_addr: uint (bits:35) = third_rw.addr;
            for k from 0 to 1 {
              if first_rw_addr == second_rw_addr and second_rw_addr == third_rw_addr {
                collision = TRUE;
                sys.global_cover.<instance'name>_collision_addr = first_rw_addr;
                break;
              };
              third_rw_addr += (third_rw.accessing_next_cache_line) ? 1 : 0;
            };
            if collision == TRUE {
              break;
            };
            second_rw_addr += (second_rw.accessing_next_cache_line) ? 1 : 0;
          };
          if collision == TRUE {
            break;
          };
          first_rw_addr += (first_rw.accessing_next_cache_line) ? 1 : 0;
        };
        if collision {
          var event_number: uint = 0;
          if first_rw.command == Write {
            event_number += 4;
          };
          if second_rw.command == Write {
            event_number += 2;
          };
          if third_rw.command == Write {
            event_number += 1;
          };
          case event_number {
            0: {emit <instance'name>_rrr_cover;};
            1: {emit <instance'name>_rrw_cover;};
            2: {emit <instance'name>_rwr_cover;};
            3: {emit <instance'name>_rww_cover;};
            4: {emit <instance'name>_wrr_cover;};
            5: {emit <instance'name>_wrw_cover;};
            6: {emit <instance'name>_wwr_cover;};
            7: {emit <instance'name>_www_cover;};
          };
        };
        collision = FALSE;
        var second_rw_addr: uint (bits:35) = second_rw.addr;
        for j from 0 to 1 {
          var third_rw_addr: uint (bits:35) = third_rw.addr;
          for k from 0 to 1 {
            if second_rw_addr == third_rw_addr {
              collision = TRUE;
              sys.global_cover.<instance'name>_collision_addr = second_rw_addr;
              break;
            };
            third_rw_addr += (third_rw.accessing_next_cache_line) ? 1 : 0;
          };
          if collision == TRUE {
            break;
          };
          second_rw_addr += (second_rw.accessing_next_cache_line) ? 1 : 0;
        };
        if collision {
          var event_number: uint = 0;
          if second_rw.command == Write {
            event_number += 2;
          };
          if third_rw.command == Write {
            event_number += 1;
          };
          case event_number {
            0: {emit <instance'name>_rr_cover;};
            1: {emit <instance'name>_rw_cover;};
            2: {emit <instance'name>_wr_cover;};
            3: {emit <instance'name>_ww_cover;};
          };
        };
      };
    };

    // Coverage code for triplets of Read/Write commands
    cover <instance'name>_rrr_cover is {
      item collision_addr_high: uint = sys.global_cover.<instance'name>_collision_addr[34:3];
      item collision_addr_low: uint = sys.global_cover.<instance'name>_collision_addr[2:1];
      cross collision_addr_high, collision_addr_low;
    };

    cover <instance'name>_rrw_cover is {
      item collision_addr_high: uint = sys.global_cover.<instance'name>_collision_addr[34:3];
      item collision_addr_low: uint = sys.global_cover.<instance'name>_collision_addr[2:1];
      cross collision_addr_high, collision_addr_low;
    };

    cover <instance'name>_rwr_cover is {
      item collision_addr_high: uint = sys.global_cover.<instance'name>_collision_addr[34:3];
      item collision_addr_low: uint = sys.global_cover.<instance'name>_collision_addr[2:1];
      cross collision_addr_high, collision_addr_low;
    };

    cover <instance'name>_rww_cover is {
      item collision_addr_high: uint = sys.global_cover.<instance'name>_collision_addr[34:3];
      item collision_addr_low: uint = sys.global_cover.<instance'name>_collision_addr[2:1];
      cross collision_addr_high, collision_addr_low;
    };

    cover <instance'name>_wrr_cover is {
      item collision_addr_high: uint = sys.global_cover.<instance'name>_collision_addr[34:3];
      item collision_addr_low: uint = sys.global_cover.<instance'name>_collision_addr[2:1];
      cross collision_addr_high, collision_addr_low;
    };

    cover <instance'name>_wrw_cover is {
      item collision_addr_high: uint = sys.global_cover.<instance'name>_collision_addr[34:3];
      item collision_addr_low: uint = sys.global_cover.<instance'name>_collision_addr[2:1];
      cross collision_addr_high, collision_addr_low;
    };

    cover <instance'name>_wwr_cover is {
      item collision_addr_high: uint = sys.global_cover.<instance'name>_collision_addr[34:3];
      item collision_addr_low: uint = sys.global_cover.<instance'name>_collision_addr[2:1];
      cross collision_addr_high, collision_addr_low;
    };

    cover <instance'name>_www_cover is {
      item collision_addr_high: uint = sys.global_cover.<instance'name>_collision_addr[34:3];
      item collision_addr_low: uint = sys.global_cover.<instance'name>_collision_addr[2:1];
      cross collision_addr_high, collision_addr_low;
    };

    // Coverage code for couplets of Read/Write commands
    cover <instance'name>_rr_cover is {
      item collision_addr_high: uint = sys.global_cover.<instance'name>_collision_addr[34:3];
      item collision_addr_low: uint = sys.global_cover.<instance'name>_collision_addr[2:1];
      cross collision_addr_high, collision_addr_low;
    };

    cover <instance'name>_rw_cover is {
      item collision_addr_high: uint = sys.global_cover.<instance'name>_collision_addr[34:3];
      item collision_addr_low: uint = sys.global_cover.<instance'name>_collision_addr[2:1];
      cross collision_addr_high, collision_addr_low;
    };

    cover <instance'name>_wr_cover is {
      item collision_addr_high: uint = sys.global_cover.<instance'name>_collision_addr[34:3];
      item collision_addr_low: uint = sys.global_cover.<instance'name>_collision_addr[2:1];
      cross collision_addr_high, collision_addr_low;
    };

    cover <instance'name>_ww_cover is {
      item collision_addr_high: uint = sys.global_cover.<instance'name>_collision_addr[34:3];
      item collision_addr_low: uint = sys.global_cover.<instance'name>_collision_addr[2:1];
      cross collision_addr_high, collision_addr_low;
    };

  };

  // Coverage that is common to tx and rx ports.
  add_coverage_on_port <instance'name>;
};

define <add_coverage_on_rx_port'statement> "add_coverage_on_rx_port <instance'name>" as {
  // Coverage that is common to tx and rx ports.
  add_coverage_on_port <instance'name>;
};

define <add_coverage_on_port'statement> "add_coverage_on_port <instance'name>" as {
  extend channel_instance_e: [<instance'name>];

  extend global_cover_t {
    <instance'name>_prev_cmd: csco_ht_packet_encoding;
    keep soft <instance'name>_prev_cmd == Nop;

    <instance'name>_interrupt_location: uint(bits:6);

    <instance'name>_available_Posted_Cmd_Buffers: uint (bits: max_buffer_counter_width);
    <instance'name>_available_NonPosted_Cmd_Buffers: uint (bits: max_buffer_counter_width);
    <instance'name>_available_Response_Cmd_Buffers: uint (bits: max_buffer_counter_width);
    <instance'name>_available_Posted_Data_Buffers: uint (bits: max_buffer_counter_width);
    <instance'name>_available_NonPosted_Data_Buffers: uint (bits: max_buffer_counter_width);
    <instance'name>_available_Response_Data_Buffers: uint (bits: max_buffer_counter_width);
    <instance'name>_Write_delay: uint;
    <instance'name>_RdResponse_delay: uint;
    <instance'name>_pending_interrupt_cmd: bool;
      keep soft <instance'name>_pending_interrupt_cmd == FALSE;
  };

  extend csco_ht_channel_interface {
    event <instance'name>_stall_cover;
    event <instance'name>_interrupt_cover;
    event <instance'name>_interrupt_stall_cover;
    event <instance'name>_available_Posted_Cmd_cover;
    event <instance'name>_available_NonPosted_Cmd_cover;
    event <instance'name>_available_Response_Cmd_cover;
    event <instance'name>_available_Posted_Data_cover;
    event <instance'name>_available_NonPosted_Data_cover;
    event <instance'name>_available_Response_Data_cover;
    event <instance'name>_Write_delay_cover;
    event <instance'name>_RdResponse_delay_cover;

    do_cover_interrupt_cmd (instance: channel_instance_e) is also {
      if instance == <instance'name> {
        sys.global_cover.<instance'name>_pending_interrupt_cmd = TRUE;
      };
    };

    do_cover_delay (instance: channel_instance_e, command_with_data: last_command_with_data_t) is also {
      if instance == <instance'name> {
        if command_with_data.command == Write {
          sys.global_cover.<instance'name>_Write_delay =
            command_with_data.data_time - command_with_data.data_length/4;
          emit <instance'name>_Write_delay_cover;
        };
        if command_with_data.command == RdResponse {
          sys.global_cover.<instance'name>_RdResponse_delay =
            command_with_data.data_time - command_with_data.data_length/4;
          emit <instance'name>_RdResponse_delay_cover;
        };
      };
    };

    // Coverage code for Write delay (number of double words interrupting Data)
    cover <instance'name>_Write_delay_cover is {
      item Write_delay: uint = sys.global_cover.<instance'name>_Write_delay;
    };

    // Coverage code for RdResponse delay (number of double words interrupting Data)
    cover <instance'name>_RdResponse_delay_cover is {
      item RdResponse_delay: uint = sys.global_cover.<instance'name>_RdResponse_delay;
    };

    // Coverage code for rx buffer counters
    do_cover_buffer (instance: channel_instance_e, virtual_channel: csco_ht_virtual_channel,
          packet_type: csco_ht_packet_type, available_buffer: uint) is also {
      if instance == <instance'name> {
        if packet_type == Ctl {
          case virtual_channel {
            Posted:    {
                sys.global_cover.<instance'name>_available_Posted_Cmd_Buffers = available_buffer;
                emit <instance'name>_available_Posted_Cmd_cover;
            };
            NonPosted: {
                sys.global_cover.<instance'name>_available_NonPosted_Cmd_Buffers = available_buffer;
                emit <instance'name>_available_NonPosted_Cmd_cover;
            };
            Response:  {
                sys.global_cover.<instance'name>_available_Response_Cmd_Buffers = available_buffer;
                emit <instance'name>_available_Response_Cmd_cover;
            };
          };
        };
        if packet_type == Data {
          case virtual_channel {
            Posted:    {
                sys.global_cover.<instance'name>_available_Posted_Data_Buffers = available_buffer;
                emit <instance'name>_available_Posted_Data_cover;
            };
            NonPosted: {
                sys.global_cover.<instance'name>_available_NonPosted_Data_Buffers = available_buffer;
                emit <instance'name>_available_NonPosted_Data_cover;
            };
            Response:  {
                sys.global_cover.<instance'name>_available_Response_Data_Buffers = available_buffer;
                emit <instance'name>_available_Response_Data_cover;
            };
          };
        };
      };
    };

    cover <instance'name>_available_Posted_Cmd_cover is {
      item available_Posted_Cmd_buffers: uint (bits: max_buffer_counter_width) =
            sys.global_cover.<instance'name>_available_Posted_Cmd_Buffers
            using illegal = (available_Posted_Cmd_buffers >
                             max_buffer_counter_size),
                  ignore =  (available_Posted_Cmd_buffers >
                             max_buffer_counter_size);
    };

    cover <instance'name>_available_NonPosted_Cmd_cover is {
      item available_NonPosted_Cmd_buffers: uint (bits: max_buffer_counter_width) =
            sys.global_cover.<instance'name>_available_NonPosted_Cmd_Buffers
            using illegal = (available_NonPosted_Cmd_buffers >
                             max_buffer_counter_size),
                  ignore =  (available_NonPosted_Cmd_buffers >
                             max_buffer_counter_size);
    };

    cover <instance'name>_available_Response_Cmd_cover is {
      item available_Response_Cmd_buffers: uint (bits: max_buffer_counter_width) =
            sys.global_cover.<instance'name>_available_Response_Cmd_Buffers
            using illegal = (available_Response_Cmd_buffers >
                             max_buffer_counter_size),
                  ignore =  (available_Response_Cmd_buffers >
                             max_buffer_counter_size);
    };

    cover <instance'name>_available_Posted_Data_cover is {
      item available_Posted_Data_buffers: uint (bits: max_buffer_counter_width) =
            sys.global_cover.<instance'name>_available_Posted_Data_Buffers
            using illegal = (available_Posted_Data_buffers >
                             max_buffer_counter_size),
                  ignore =  (available_Posted_Data_buffers >
                             max_buffer_counter_size);
    };

    cover <instance'name>_available_NonPosted_Data_cover is {
      item available_NonPosted_Data_buffers: uint (bits: max_buffer_counter_width) =
            sys.global_cover.<instance'name>_available_NonPosted_Data_Buffers
            using illegal = (available_NonPosted_Data_buffers >
                             max_buffer_counter_size),
                  ignore =  (available_NonPosted_Data_buffers >
                             max_buffer_counter_size);
    };

    cover <instance'name>_available_Response_Data_cover is {
      item available_Response_Data_buffers: uint (bits: max_buffer_counter_width) =
            sys.global_cover.<instance'name>_available_Response_Data_Buffers
            using illegal = (available_Response_Data_buffers >
                             max_buffer_counter_size),
                  ignore =  (available_Response_Data_buffers >
                             max_buffer_counter_size);
    };

    // Coverage code for interrupts/stalls
    do_cover_stall (instance: channel_instance_e, location: uint (bits:6),
                    command_data_b: bool, data_stalled: bool) is also {
      if instance == <instance'name> {
        sys.global_cover.<instance'name>_interrupt_location = location;
        if command_data_b {
          emit <instance'name>_stall_cover;
          if data_stalled {
            emit <instance'name>_interrupt_stall_cover;
          };
        } else {
            emit <instance'name>_interrupt_cover;
        };
      };
    };

    // coverage for command packet stalls while no data packet is pending
    cover <instance'name>_stall_cover is {
      item stall_location: uint(bits:6) =
        sys.global_cover.<instance'name>_interrupt_location using
          ranges = {
            range ([4], "4");
          };
    };

    // coverage for command packet stalls while some data packet is pending
    cover <instance'name>_interrupt_stall_cover is {
      item interrupt_stall_location: uint(bits:6) =
        sys.global_cover.<instance'name>_interrupt_location using
          ranges = {
            range ([4], "4");
          };
    };

    // coverage for data packet interrupts
    cover <instance'name>_interrupt_cover is {
      item interrupt_location: uint(bits:6) = sys.global_cover.<instance'name>_interrupt_location using
          ranges = {
            range ([0], "0");
            range ([4], "4");
            range ([8], "8");
            range ([12], "12");
            range ([16], "16");
            range ([20], "20");
            range ([24], "24");
            range ([28], "28");
            range ([32], "32");
            range ([36], "36");
            range ([40], "40");
            range ([44], "44");
            range ([48], "48");
            range ([52], "52");
            range ([56], "56");
            range ([60], "60");
          };
    };
  };

  extend csco_ht_packet {
    event <instance'name>_nop_packet_cover;
    event <instance'name>_flush_packet_cover;
    event <instance'name>_write_byte_packet_cover;
    event <instance'name>_write_dword_packet_cover;
    event <instance'name>_read_byte_packet_cover;
    event <instance'name>_read_dword_packet_cover;
    event <instance'name>_fence_packet_cover;
    event <instance'name>_broadcast_packet_cover;
    event <instance'name>_tgtdone_packet_cover;
    event <instance'name>_rdresponse_packet_cover;
    event <instance'name>_interrupt_packet_cover;
    event <instance'name>_interrupt_ack_packet_cover;
    event <instance'name>_byte_write_packet_cover;
    event <instance'name>_packet_ordering_cover;
    event <instance'name>_interrupt_destination_cover;
  };

  extend <instance'name> csco_ht_packet {
    do_cover() is {
      emit <instance'name>_packet_ordering_cover;
      sys.global_cover.<instance'name>_prev_cmd = command;
      case command {
        Nop:        { emit <instance'name>_nop_packet_cover; };
        Flush:      { emit <instance'name>_flush_packet_cover; };

        Write:      {
          if me is a Write csco_ht_packet (p) and p.data_type == Byte {
            if p.interrupt == FALSE {
              emit <instance'name>_write_byte_packet_cover;
            } else {
              emit <instance'name>_interrupt_packet_cover;
            };
          } else {
            emit <instance'name>_write_dword_packet_cover;
          };
        };
        Read:       {
          if me is a Read csco_ht_packet (p) and p.data_type == Byte {
            emit <instance'name>_read_byte_packet_cover;
          } else {
            emit <instance'name>_read_dword_packet_cover;
          };
        };
        RdResponse: { emit <instance'name>_rdresponse_packet_cover; };
        TgtDone:    { emit <instance'name>_tgtdone_packet_cover; };
        Broadcast:  {
          if me is a Broadcast csco_ht_packet (p) and p.interrupt == FALSE {
            emit <instance'name>_broadcast_packet_cover;
          } else {
            emit <instance'name>_interrupt_ack_packet_cover;
          };
        };
        Fence:      { emit <instance'name>_fence_packet_cover; };
        Byte_Write: {
          if sys.global_cover.<instance'name>_pending_interrupt_cmd == FALSE {
            emit <instance'name>_byte_write_packet_cover;
          } else {
            emit <instance'name>_interrupt_destination_cover;
            sys.global_cover.<instance'name>_pending_interrupt_cmd = FALSE;
          };
        };
      };
    };
  };

  extend csco_ht_packet {

    // Coverage code for couplet commands following each other
    cover <instance'name>_packet_ordering_cover is {
      item next_cmd: csco_ht_packet_encoding = command;
      item prev_cmd: csco_ht_packet_encoding = sys.global_cover.<instance'name>_prev_cmd;
      cross next_cmd, prev_cmd;
    };

    // Coverage code for Nop commands
    when Nop'command csco_ht_packet {
      cover <instance'name>_nop_packet_cover is {
        item PostCmd;
        item PostData;
        item Response;
        item ResponseData;
        item NonPostCmd;
        item NonPostData;
      };
    };

    // Coverage code for Flush commands
    when Flush'command csco_ht_packet {
      cover <instance'name>_flush_packet_cover is {
        item SeqID;
        item UnitID;
        item PassPW;
        item SrcTag;
      };
    };

    // Coverage code for Write commands
    when Write'command csco_ht_packet {

      // Coverage code for Byte Write commands
      cover <instance'name>_write_byte_packet_cover is {
        item BW_latency_req;
        item access_type;
        item vChan using ignore = (vChan == NA or vChan == Response);
        item passing_rule using ignore = (passing_rule == NA);
        item SeqID;
        item UnitID;
        item SrcTag_Rsv;
        item Compat;
        item Count: uint (bits:4) = Mask_Count;

        cross vChan, SrcTag_Rsv;

        // Addr
        item region : uint (bits: 16) = Addr[39:24];
        item addr   : uint (bits: 24) = Addr[23:0];
        item cache_line_byte : uint (bits: 5) = Addr[4:0] using
          ignore = ((interrupt == TRUE) or (cache_line_byte[1:0] != 0));

        item interrupt;

        item packet_length: uint = (Mask_Count * 4) using // excluding the byte mask!
          ranges = {
            range ([4], "4");
            range ([8], "8");
            range ([12], "12");
            range ([16], "16");
            range ([20], "20");
            range ([24], "24");
            range ([28], "28");
            range ([32], "32");
            range ([36], "36");
            range ([40], "40");
            range ([44], "44");
            range ([48], "48");
            range ([52], "52");
            range ([56], "56");
            range ([60], "60");
            range ([64], "64");
          };

        cross cache_line_byte, packet_length using
          illegal = ((cache_line_byte + packet_length) > 32),
          ignore = ((cache_line_byte + packet_length) > 32);
      };

      // Coverage code for DoubleWord Write commands
      cover <instance'name>_write_dword_packet_cover is {
        item BW_latency_req;
        item access_type;
        item vChan using ignore = (vChan == NA or vChan == Response);
        item passing_rule using ignore = (passing_rule == NA);
        item SeqID;
        item UnitID;
        item SrcTag_Rsv;
        item Compat;
        item Count: uint (bits:4) = Mask_Count;

        cross vChan, SrcTag_Rsv;

        // Addr
        item region : uint (bits: 16) = Addr[39:24];
        item addr   : uint (bits: 24) = Addr[23:0];
        item cache_line_byte : uint (bits: 6) = Addr[5:0] using
          ignore = (cache_line_byte[1:0] != 0);

        item packet_length: uint = ((Mask_Count + 1) * 4) using
          ranges = {
            range ([4], "4");
            range ([8], "8");
            range ([12], "12");
            range ([16], "16");
            range ([20], "20");
            range ([24], "24");
            range ([28], "28");
            range ([32], "32");
            range ([36], "36");
            range ([40], "40");
            range ([44], "44");
            range ([48], "48");
            range ([52], "52");
            range ([56], "56");
            range ([60], "60");
            range ([64], "64");
          };

        cross cache_line_byte, packet_length using
          illegal = ((cache_line_byte + packet_length) > 64),
          ignore = ((cache_line_byte + packet_length) > 64);
      };

      // Coverage code for Interrupt commands
      cover <instance'name>_interrupt_packet_cover is {
        item BW_latency_req;
        item access_type;
        item passing_rule using ignore = (passing_rule == NA);
        item SeqID;
        item UnitID;

        item MT using ignore = (MT == EOI);
        item TM;
        item DM;
        item IntrDest;
        item Vector;
      };

    };

    // Coverage code for the Write byte mask permutations
    when Byte_Write'command csco_ht_packet {
      cover <instance'name>_byte_write_packet_cover is {
        // NOTE::::::::: The Mask field of a Byte_Write packet is not a Mask for interrupt packets!
        // It carries interrupt information
        item mask0: bit = Mask[0:0];
        item mask1: bit = Mask[1:1];
        item mask2: bit = Mask[2:2];
        item mask3: bit = Mask[3:3];
        item mask4: bit = Mask[4:4];
        item mask5: bit = Mask[5:5];
        item mask6: bit = Mask[6:6];
        item mask7: bit = Mask[7:7];
        item mask8: bit = Mask[8:8];
        item mask9: bit = Mask[9:9];
        item mask10: bit = Mask[10:10];
        item mask11: bit = Mask[11:11];
        item mask12: bit = Mask[12:12];
        item mask13: bit = Mask[13:13];
        item mask14: bit = Mask[14:14];
        item mask15: bit = Mask[15:15];
        item mask16: bit = Mask[16:16];
        item mask17: bit = Mask[17:17];
        item mask18: bit = Mask[18:18];
        item mask19: bit = Mask[19:19];
        item mask20: bit = Mask[20:20];
        item mask21: bit = Mask[21:21];
        item mask22: bit = Mask[22:22];
        item mask23: bit = Mask[23:23];
        item mask24: bit = Mask[24:24];
        item mask25: bit = Mask[25:25];
        item mask26: bit = Mask[26:26];
        item mask27: bit = Mask[27:27];
        item mask28: bit = Mask[28:28];
        item mask29: bit = Mask[29:29];
        item mask30: bit = Mask[30:30];
        item mask31: bit = Mask[31:31];
      };

      cover <instance'name>_interrupt_destination_cover is {
        item interrupt_destination: uint(bits:24) = Mask[23:0];
      };
    };

    // Coverage code for Read commands
    when Read'command csco_ht_packet {

      // Coverage for Byte Read commands
      cover <instance'name>_read_byte_packet_cover is {
        item passing_rule using ignore = (passing_rule == NA);
        item response_passing_rule using ignore = (response_passing_rule == NA);
        item data_type;
        item BW_latency_req;
        item access_type;

        // Addr
        item region : uint (bits: 16) = Addr[39:24];
        item addr   : uint (bits: 24) = Addr[23:0];

        item SeqID;
        item UnitID;
        item SrcTag_Rsv;
        item Compat;
        item Mask: uint (bits:4) = Mask_Count;
      };

      // Coverage for DoubleWord Read commands
      cover <instance'name>_read_dword_packet_cover is {
        item passing_rule using ignore = (passing_rule == NA);
        item response_passing_rule using ignore = (response_passing_rule == NA);
        item data_type;
        item BW_latency_req;
        item access_type;

        // Addr
        item region : uint (bits: 16) = Addr[39:24];
        item addr   : uint (bits: 24) = Addr[23:0];
        item cache_line_byte : uint (bits: 6) = Addr[5:0] using
          ignore = (cache_line_byte[1:0] != 0);

        item SeqID;
        item UnitID;
        item SrcTag_Rsv;
        item Compat;
        item Count: uint (bits:4) = Mask_Count;

        item packet_length: uint = ((Mask_Count + 1) * 4) using
          ranges = {
            range ([4], "4");
            range ([8], "8");
            range ([12], "12");
            range ([16], "16");
            range ([20], "20");
            range ([24], "24");
            range ([28], "28");
            range ([32], "32");
            range ([36], "36");
            range ([40], "40");
            range ([44], "44");
            range ([48], "48");
            range ([52], "52");
            range ([56], "56");
            range ([60], "60");
            range ([64], "64");
          };

        cross cache_line_byte, packet_length using
          illegal = ((cache_line_byte + packet_length) > 64),
          ignore = ((cache_line_byte + packet_length) > 64);
      };
    };

    // Coverage code for RdResponse commands
    when RdResponse'command csco_ht_packet {
      cover <instance'name>_rdresponse_packet_cover is {
        item passing_rule using ignore = (passing_rule == NA);
        item UnitID;
        item Bridge;
        item SrcTag;
        item ErrorFlag;
        item Count;
        item NXA;

        item packet_length : uint = ((Count + 1) * 4) using
          ranges = {
            range ([4], "4");
            range ([8], "8");
            range ([12], "12");
            range ([16], "16");
            range ([20], "20");
            range ([24], "24");
            range ([28], "28");
            range ([32], "32");
            range ([36], "36");
            range ([40], "40");
            range ([44], "44");
            range ([48], "48");
            range ([52], "52");
            range ([56], "56");
            range ([60], "60");
            range ([64], "64");
          };
        item iostream: uint (bits:5) = IOStream();
      };
    };

    // Coverage code for TgtDone commands
    when TgtDone'command csco_ht_packet {
      cover <instance'name>_tgtdone_packet_cover is {
        item passing_rule using ignore = (passing_rule == NA);
        item UnitID;
        item Bridge;
        item SrcTag;
        item ErrorFlag;
        item NXA;

        item iostream: uint (bits:5) = IOStream();
      };
    };

    // Coverage code for Broadcast commands
    when Broadcast'command csco_ht_packet {
      cover <instance'name>_broadcast_packet_cover is {
        item vChan using ignore = (vChan == NA or vChan == Response);
        item passing_rule using ignore = (passing_rule == NA);
        item interrupt;

        item SeqID;
        item UnitID;
        item SrcTag_Rsv;

        cross vChan, SrcTag_Rsv;

        // Addr
        item region : uint (bits: 16) = Addr[39:24];
        item addr   : uint (bits: 24) = Addr[23:0];
      };

      // Coverage code for Interrupt Ack commands (broadcast)
      cover <instance'name>_interrupt_ack_packet_cover is {
        item passing_rule using ignore = (passing_rule == NA);
        item SeqID;
        item UnitID;

        item MT;
        item Vector;
      };

    };

    // Coverage for Fence commands
    when Fence'command csco_ht_packet {
      cover <instance'name>_fence_packet_cover is {
        item passing_rule using ignore = (passing_rule == NA);

        item SeqID;
        item UnitID; // IOStream is the same thing
      };
    };
  };
};

'>
