//--------------------------------------------------------------------------
// Title         : HT Sample Bench
// Project       : HT Testbench
//--------------------------------------------------------------------------
// File          : csco_ht_sample_bench.e
// Author        : Brian Van Essen  <vanessen@cisco.com>
// Created       : 16.03.2001
// Last modified : 16.03.2001
//--------------------------------------------------------------------------
// Description :
// This file is an example of a top level testbench file that provides system
// topology.
//--------------------------------------------------------------------------
// Copyright (c) 2001 by Cisco Systems, Inc.. This model is the confidential and
// proprietary property of Cisco Systems, Inc. and the possession or use of this
// file requires a written license from Cisco Systems, Inc..
//--------------------------------------------------------------------------
// Modification history :
// 16.03.2001 : created
//-------------------------------------------------------------------------

<'

//-----------------------------------------------------------------------------
// HT Topology (Instance declaration)
//-----------------------------------------------------------------------------

extend sys {

  init() is also {
    set_config(print,scale, ns);                           // Tell Debug Messages to print time unit in Nanoseconds
  };

  when FALSE'csco_ht_development_test sys {

    // Instantiate all of the HT devices that Specman will model on the chain.
    //
    // Define a Host Bridge model of what could be the BC12500 and provide the appropriate
    // parameters.
    //

    BC12500 : csco_ht_device is instance;
      keep BC12500.device_function == Host_Bridge;

    //
    // Device Level Parameters
    //

      // Set up the HDL path to the interface point for this device.
      keep BC12500.hdl_path() == "wasabi_tb";

      // System wide Power and Reset signals
      keep BC12500.Reset_L == "csco_ht_RESET_NEG";
      keep BC12500.PowerOk == "csco_ht_POWEROK";

    //
    // Define the signal interface for the link.
    //

    keep for each (interface) in BC12500.csco_ht_interfaces {

      //
      // Define the basic clock for the link interface
      //

      interface.csco_ht_link.csco_ht_link_fundamental_clock_signal == appendf("~/wasabi_tb.csco_ht_BASE_CLK");

      //
      // Define the signals for the DUT's receive path (The 'e' device's TX path)
      //

      // clk buses
      interface.csco_ht_link.clk_tx_H == "Proc_csco_ht_RX_CLK_H";
      interface.csco_ht_link.clk_tx_L == "Proc_csco_ht_RX_CLK_L";
      interface.csco_ht_link.ctl_tx_H == "Proc_csco_ht_RX_CTL_H";
      interface.csco_ht_link.ctl_tx_L == "Proc_csco_ht_RX_CTL_L";

      // cad buses
      interface.csco_ht_link.cad_tx_H == "Proc_csco_ht_RX_CAD_H";
      interface.csco_ht_link.cad_tx_L == "Proc_csco_ht_RX_CAD_L";

      //
      // Define the signals for the DUT's transmit path (The 'e' device's RX path)
      //

      // clk buses
      interface.csco_ht_link.clk_rx_H == "Proc_csco_ht_TX_CLK_H";
      interface.csco_ht_link.clk_rx_L == "Proc_csco_ht_TX_CLK_L";
      interface.csco_ht_link.ctl_rx_H == "Proc_csco_ht_TX_CTL_H";
      interface.csco_ht_link.ctl_rx_L == "Proc_csco_ht_TX_CTL_L";

      // cad buses
      interface.csco_ht_link.cad_rx_H == "Proc_csco_ht_TX_CAD_H";
      interface.csco_ht_link.cad_rx_L == "Proc_csco_ht_TX_CAD_L";

    };                                                     // End Keep for each

    //
    // End definition of BC12500 Host Bridge Device.
    //


    //
    // Define a EOC Device that will sit on the chain after the Tunnel device and provide
    // the appropriate parameters.
    //
    io_device_0 : csco_ht_device is instance;
      keep io_device_0.device_function == EOC_IO_Device;
    //
    // Device Level Parameters
    //
    // Set up the HDL path to the interface point for this device.
      keep io_device_0.hdl_path() == "wasabi_tb";
    // System wide Power and Reset signals
      keep io_device_0.Reset_L == "csco_ht_RESET_NEG";
    keep io_device_0.PowerOk == "csco_ht_POWEROK";

    //
    // Define the signal interface for the link.
    //
    keep for each (interface) in io_device_0.csco_ht_interfaces {

      //
      // Define the basic clock for the link interface
      //
      interface.csco_ht_link.csco_ht_link_fundamental_clock_signal == appendf("~/wasabi_tb.csco_ht_BASE_CLK");

      //
      // Define the signals for the DUT's receive path (The 'e' device's TX path)
      //

      // clk buses
      interface.csco_ht_link.clk_tx_H == "Fwd1_csco_ht_RX_CLK_H";
      interface.csco_ht_link.clk_tx_L == "Fwd1_csco_ht_RX_CLK_L";
      interface.csco_ht_link.ctl_tx_H == "Fwd1_csco_ht_RX_CTL_H";
      interface.csco_ht_link.ctl_tx_L == "Fwd1_csco_ht_RX_CTL_L";

      // cad buses
      interface.csco_ht_link.cad_tx_H == "Fwd1_csco_ht_RX_CAD_H";
      interface.csco_ht_link.cad_tx_L == "Fwd1_csco_ht_RX_CAD_L";

      //
      // Define the signals for the DUT's transmit path (The 'e' device's RX path)
      //

      // clk buses
      interface.csco_ht_link.clk_rx_H == "Fwd1_csco_ht_TX_CLK_H";
      interface.csco_ht_link.clk_rx_L == "Fwd1_csco_ht_TX_CLK_L";
      interface.csco_ht_link.ctl_rx_H == "Fwd1_csco_ht_TX_CTL_H";
      interface.csco_ht_link.ctl_rx_L == "Fwd1_csco_ht_TX_CTL_L";

      // cad buses
      interface.csco_ht_link.cad_rx_H == "Fwd1_csco_ht_TX_CAD_H";
      interface.csco_ht_link.cad_rx_L == "Fwd1_csco_ht_TX_CAD_L";
    };

    //
    // End definition of io_device_0 EOC IO Device.
    //
  };

  //
  // For every verilog wire that the HT test bench is going to drive, there needs
  // to be a verilog variables statement below.  Additionally if any of these are
  // changed, then a new stubs file must be generated.
  // For defining new signals, the format is ~/top_level.hierarchy.signal_name
  // These are necessary for VCS but not NC-Verilog
  //

  // Processor HT Rx Interface
  verilog variable '~/wasabi_tb.Proc_csco_ht_RX_CAD_H' using wire;
  verilog variable '~/wasabi_tb.Proc_csco_ht_RX_CAD_L' using wire;
  verilog variable '~/wasabi_tb.Proc_csco_ht_RX_CTL_H' using wire;
  verilog variable '~/wasabi_tb.Proc_csco_ht_RX_CTL_L' using wire;
  verilog variable '~/wasabi_tb.Proc_csco_ht_RX_CLK_H' using wire;
  verilog variable '~/wasabi_tb.Proc_csco_ht_RX_CLK_L' using wire;

  // Forward path 1 HT Rx Interface
  verilog variable '~/wasabi_tb.Fwd1_csco_ht_RX_CAD_H' using wire;
  verilog variable '~/wasabi_tb.Fwd1_csco_ht_RX_CAD_L' using wire;
  verilog variable '~/wasabi_tb.Fwd1_csco_ht_RX_CTL_H' using wire;
  verilog variable '~/wasabi_tb.Fwd1_csco_ht_RX_CTL_L' using wire;
  verilog variable '~/wasabi_tb.Fwd1_csco_ht_RX_CLK_H' using wire;
  verilog variable '~/wasabi_tb.Fwd1_csco_ht_RX_CLK_L' using wire;
  //
};

//
// If the user is running a standard test, this must be uncommented and setup so that
// the DUT will provide a signal that will indicate to start the test at the signal.
//
// extend csco_ht_test_control_block {
//                                                         // An event to indicate that the HT system can start generating traffic.
//   event csco_ht_start_traffic is only change('~/wasabi_tb.start_csco_ht_traffic')@sim;
// };
//
// System Level Flags
//


extend sys {
  // Flags to allow the user to have the testbench stop immediately if an error is found.
  keep csco_ht_errors.initialization_error_severity == Error_Stop;
  //  keep csco_ht_errors.protocol_error_severity == Error_Stop;
  keep csco_ht_errors.protocol_error_severity == Error;
  keep csco_ht_errors.transaction_error_severity == Error_Stop;

  // This is a flag that will indicate that the clock provided to the 'e' model is clock that is running at twice
  // the desired link frequency.  One effect of this is that the model will not be able to dynamically change link
  // speed during the simulation.  Any link frequency shift will have to be done by the clock source.
  // If this flag is false, then a clock that is 1/12th ns needs to be provided so that it can be multiplied to achieve
  // any one of the legal HT values.  (i.e. 200MHz, 300MHz, 400MHz, 500MH, 600MHz, 800MHz, 1000MHz).
  keep csco_ht_use_direct_csco_ht_clock == TRUE;

  // Status flag used to indicate if the test run was successful or if errors occurred.
  test_passed : bool;
    keep soft test_passed == FALSE;

  //
  // Watchdog TCM and monitoring for errors.

  monitor_test_progress()@sys.csco_ht_test_control.csco_ht_watchdog_clock is {
    // Check for the csco_ht_model_done flag or the csco_ht_model_inactive flag to be set.
    wait true(csco_ht_test_control.csco_ht_model_done == TRUE || csco_ht_test_control.csco_ht_model_inactive == TRUE
              || csco_ht_test_control.csco_ht_model_error == TRUE);

    if(csco_ht_test_control.csco_ht_model_outstanding_transactions != 0) {
      // The model has encountered a fatal error.
      outf("********************************************************************************\n");
      outf("HT Model has outstanding transactions\n");
      outf("********************************************************************************\n");
    };

    outf("********************************************************************************\n");

    if(csco_ht_test_control.csco_ht_model_error == TRUE) {
      // The model has encountered a fatal error.
      test_passed = FALSE;
      outf("HT Model Error\n");
    }else if(csco_ht_test_control.csco_ht_model_done == TRUE && csco_ht_test_control.csco_ht_model_inactive == FALSE) {
      // The model has successfully completed.
      test_passed = TRUE;
      outf("HT Model Completed\n");
    }else if(csco_ht_test_control.csco_ht_model_done == FALSE && csco_ht_test_control.csco_ht_model_inactive == TRUE) {
      // The model is inactive and has not completed traffic generation.
      test_passed = FALSE;
      outf("HT Model Inactive\n");
    }else {
      dut_error("The HT model has set both the csco_ht_model_done flag and the csco_ht_model_inactive, which is illegal.");
    };
    outf("********************************************************************************\n");

    stop_run();
  };

  //
  // Once the test run is completed check to see if any errors occurred.  If so, dump the results and set the status flag.

  finalize() is also {
    test_passed = csco_ht_errors.report_summary();
    outf("\n");
    outf("********************************************************************************\n");
    if(test_passed == FALSE) {
      outf("HT Test FAILED ");
    }else {
      outf("HT Test PASSED ");
    };
    outf("\n");
    outf("********************************************************************************\n");
  };

  //
  // Start the watchdog.
  run() is also {
    start monitor_test_progress();
  };
};

//
// Provide some general parameters to the global environment.
//

extend global {
  //
  setup_test() is also {

    set_config(print, radix, hex, items, 8192, list_lines, 8192, line_size, 120);
    //    set_config(gen, max_structs, 5000000, seed, random);

    set_config(memory, gc_threshold, 50000000);
    set_config(memory, gc_increment, 20000000);

    set_config(memory, max_size, 300M);
    set_config(memory, absolute_max_size, 310M);

    set_config(cover, mode, normal);

    set_config(run, tick_max, 500000000000);

    specman("set wave virsim");
  };
  //
  // Provide an example of how a test bench should monitor system performance.
  //
  finalize_test() is also {
    var output_lines                  : list of string;
    var fp                            : file;

    outf("NOTE %d seconds real time expired during simulation\n",session.user_time);
    outf("NOTE %d ticks expired during simulation\n",sys.time);
    outf("NOTE Simulation Performance: %d Clock Cycles per Second\n\n",sys.time/session.user_time);

//     fp = files.open("audit.trail", "a", "Text file");
//     files.write(fp,appendf("\n%12d ns (TEST_STATS): NOTE %d seconds real time expired during simulation",sys.time,session.user_time));
//     files.write(fp,appendf("%12d ns (TEST_STATS): NOTE %d QED Clock Cycles Expired during simulation",sys.time,sys.time/160));
//     files.write(fp,appendf("%12d ns (TEST_STATS): NOTE Simulation Performance: %d Clock Cycles per Second\n",sys.time,sys.time/160/session.user_time));
//     files.close(fp);
  };                                                       // End Extend finalize test
  //
};


'>

//
// Below are three examples of HT devices.  One Host Bridge, one Tunnel and one EOC device.
//

  //
  // Instantiate a basic Host Bridge device.
  //
  example_host : csco_ht_device is instance;
    keep example_host.device_function == Host_Bridge;
  //
  // Device Level Parameters
  //

  // Set up the HDL path to the interface point for this device.
    keep example_host.hdl_path() == "testbench";

  // System wide Power and Reset signals
    keep example_host.Reset_L == "csco_ht_RESET_NEG";
    keep example_host.PowerOk == "csco_ht_POWEROK";

  //
  // Define the signal interface for the link.
  //

  keep for each (interface) in example_host.csco_ht_interfaces {

    //
    // Define the basic clock for the link interface
    //
    interface.csco_ht_link.csco_ht_link_fundamental_clock_signal == appendf("csco_ht_CLK");

    //
    // Define the signals for the DUT's receive path (The 'e' device's TX path)
    //

    for each in interface.csco_ht_link.clk_tx_H {
      it == appendf("csco_ht_RX_CLK_H");
    };
    for each in interface.csco_ht_link.clk_tx_L {
      it == appendf("csco_ht_RX_CLK_L");
    };
    interface.csco_ht_link.ctl_tx_H == "csco_ht_RX_CTL_H";
    interface.csco_ht_link.ctl_tx_L == "csco_ht_RX_CTL_L";
    for each in interface.csco_ht_link.cad_tx_H {
      it == appendf("csco_ht_RX_CAD%d_H", index);
    };
    for each in interface.csco_ht_link.cad_tx_L {
      it == appendf("csco_ht_RX_CAD%d_L", index);
    };

    //
    // Define the signals for the DUT's transmit path (The 'e' device's RX path)
    //

    for each in interface.csco_ht_link.clk_rx_H {
      it == appendf("csco_ht_TX_CLK_H");
    };
    for each in interface.csco_ht_link.clk_rx_L {
      it == appendf("csco_ht_TX_CLK_L");
    };
    interface.csco_ht_link.ctl_rx_H == "csco_ht_TX_CTL_H";
    interface.csco_ht_link.ctl_rx_L == "csco_ht_TX_CTL_L";
    for each in interface.csco_ht_link.cad_rx_H {
      it == appendf("csco_ht_TX_CAD%d_H", index);
    };
    for each in interface.csco_ht_link.cad_rx_L {
      it == appendf("csco_ht_TX_CAD%d_L", index);
    };

  //
  // End definition of basic Host Bridge Device.
  //

  //
  // Instantiate a basic Tunnel IO device.
  //
  example_tunnel : csco_ht_device is instance;
    keep example_tunnel.device_function == Tunnel_IO_Device;

  //
  // Device Level Parameters
  //

  // Set up the HDL path to the interface point for this device.
    keep example_tunnel.hdl_path() == "wasabi_tb";

  // System wide Power and Reset signals
    keep example_tunnel.Reset_L == "csco_ht_RESET_NEG";
    keep example_tunnel.PowerOk == "csco_ht_POWEROK";

  //
  // Define the signal interface for the link.
  // A tunnel has two interfaces so the definition of signals is more complicated.
  // a preface of "index == 0 =>" implies that the first interface should have the
  // specified value and a preface of "index == 1 =>" that the second interface
  // should have the specified value.
  //

  keep for each (interface) using index (interface_index) in example_tunnel.csco_ht_interfaces {
    //
    // Define the basic clock for the link interface
    //
    interface.csco_ht_link.csco_ht_link_fundamental_clock_signal == appendf("csco_ht_CLK");

    //
    // Define the signals for the DUT's receive path (The 'e' device's TX path)
    //
    for each in interface.csco_ht_link.clk_tx_H {
      interface_index == 0 => it == appendf("csco_ht_0_RX_CLK_H");
      interface_index == 1 => it == appendf("csco_ht_1_RX_CLK_H");
    };
    for each in interface.csco_ht_link.clk_tx_L {
      interface_index == 0 => it == appendf("csco_ht_0_RX_CLK_L");
      interface_index == 1 => it == appendf("csco_ht_1_RX_CLK_L");
    };
    interface_index == 0 => interface.csco_ht_link.ctl_tx_H == "csco_ht_0_RX_CTL_H";
    interface_index == 1 => interface.csco_ht_link.ctl_tx_H == "csco_ht_1_RX_CTL_H";
    interface_index == 0 => interface.csco_ht_link.ctl_tx_L == "csco_ht_0_RX_CTL_L";
    interface_index == 1 => interface.csco_ht_link.ctl_tx_L == "csco_ht_1_RX_CTL_L";
    for each in interface.csco_ht_link.cad_tx_H {
      interface_index == 0 => it == appendf("csco_ht_0_RX_CAD%d_H", index);
      interface_index == 1 => it == appendf("csco_ht_1_RX_CAD%d_H", index);
    };
    for each in interface.csco_ht_link.cad_tx_L {
      interface_index == 0 => it == appendf("csco_ht_0_RX_CAD%d_L", index);
      interface_index == 1 => it == appendf("csco_ht_1_RX_CAD%d_L", index);
    };

    //
    // Define the signals for the DUT's transmit path (The 'e' device's RX path)
    //
    for each in interface.csco_ht_link.clk_rx_H {
      interface_index == 0 => it == appendf("csco_ht_0_TX_CLK_H");
      interface_index == 1 => it == appendf("csco_ht_1_TX_CLK_H");
    };
    for each in interface.csco_ht_link.clk_rx_L {
      interface_index == 0 => it == appendf("csco_ht_0_TX_CLK_L");
      interface_index == 1 => it == appendf("csco_ht_1_TX_CLK_L");
    };
    interface_index == 0 => interface.csco_ht_link.ctl_rx_H == "csco_ht_0_TX_CTL_H";
    interface_index == 1 => interface.csco_ht_link.ctl_rx_H == "csco_ht_1_TX_CTL_H";
    interface_index == 0 => interface.csco_ht_link.ctl_rx_L == "csco_ht_0_TX_CTL_L";
    interface_index == 1 => interface.csco_ht_link.ctl_rx_L == "csco_ht_1_TX_CTL_L";
    for each in interface.csco_ht_link.cad_rx_H {
      interface_index == 0 => it == appendf("csco_ht_0_TX_CAD%d_H", index);
      interface_index == 1 => it == appendf("csco_ht_1_TX_CAD%d_H", index);
    };
    for each in interface.csco_ht_link.cad_rx_L {
      interface_index == 0 => it == appendf("csco_ht_0_TX_CAD%d_L", index);
      interface_index == 1 => it == appendf("csco_ht_1_TX_CAD%d_L", index);
    };
  //
  // End definition of basic Tunnel IO Device.
  //

  //
  // Instantiate a basic EOC IO device.
  //
  example_EOC : csco_ht_device is instance;

    keep example_EOC.device_function == EOC_IO_Device;

  //
  // Device Level Parameters
  //
  // Set up the HDL path to the interface point for this device.
  keep example_EOC.hdl_path() == "testbench";

  // System wide Power and Reset signals
  keep example_EOC.Reset_L == "csco_ht_RESET_NEG";
  keep example_EOC.PowerOk == "csco_ht_POWEROK";

  //
  // Define the signal interface for the link.
  //
  keep for each (interface) in example_EOC.csco_ht_interfaces {
    //
    // Define the basic clock for the link interface
    //
    interface.csco_ht_link.csco_ht_link_fundamental_clock_signal == appendf("csco_ht_CLK");

    //
    // Define the signals for the DUT's receive path (The 'e' device's TX path)
    //
    for each in interface.csco_ht_link.clk_tx_H {
      it == appendf("csco_ht_RX_CLK_H");
    };
    for each in interface.csco_ht_link.clk_tx_L {
      it == appendf("csco_ht_RX_CLK_L");
    };
    interface.csco_ht_link.ctl_tx_H == "csco_ht_RX_CTL_H";
    interface.csco_ht_link.ctl_tx_L == "csco_ht_RX_CTL_L";
    for each in interface.csco_ht_link.cad_tx_H {
      it == appendf("csco_ht_RX_CAD%d_H", index);
    };
    for each in interface.csco_ht_link.cad_tx_L {
      it == appendf("csco_ht_RX_CAD%d_L", index);
    };

    //
    // Define the signals for the DUT's transmit path (The 'e' device's RX path)
    //
    for each in interface.csco_ht_link.clk_rx_H {
      it == appendf("csco_ht_TX_CLK_H");
    };
    for each in interface.csco_ht_link.clk_rx_L {
      it == appendf("csco_ht_TX_CLK_L");
    };
    interface.csco_ht_link.ctl_rx_H == "csco_ht_TX_CTL_H";
    interface.csco_ht_link.ctl_rx_L == "csco_ht_TX_CTL_L";
    for each in interface.csco_ht_link.cad_rx_H {
      it == appendf("csco_ht_TX_CAD%d_H", index);
    };
    for each in interface.csco_ht_link.cad_rx_L {
      it == appendf("csco_ht_TX_CAD%d_L", index);
    };

  //
  // End definition of basic EOC IO Device.
  //
