//-----------------------------------------------------------------------------
// Title         : HT Sample Test
// Project       : HT Testbench
//-----------------------------------------------------------------------------
// File          : csco_ht_sample_test.e
// Author        : Brian Van Essen  <vanessen@cisco.com>
// Created       : 25.01.2001
// Last modified : 04.05.2001
//-----------------------------------------------------------------------------
// Description :
// This file is an example of a top level test file that provides traffic definitions
// and test specific configuration.
//-----------------------------------------------------------------------------
// Copyright (c) 2001 by Cisco Systems, Inc.. This model is the confidential and
// proprietary property of Cisco Systems, Inc. and the possession or use of this
// file requires a written license from Cisco Systems, Inc..
//------------------------------------------------------------------------------
// Modification history :
// 25.01.2001 : created
// 17.07.2002 : Modified for ERM Message Usage
//-----------------------------------------------------------------------------

// This test file contains all of the constraints that the user should adjust to customize their test.
// Note: Any generation constraint outside of this list is intended to keep the generator from creating illegal/invalid
// test cases.

<'

import evc_util/e/evc_util_top;
import ../e/csco_ht_top;
import csco_ht_sample_bench;

// Define the verilog time scale for this model
verilog time 1ps / 1ps;

//----------------------------------------------------------------------------
// Define System Topology
// This example has an 'e' Host Bridge and EOC Device surrounding a verilog Tunnel
// device which is the DUT.
//------------------------------------------------------------------------------

extend sys {

  when FALSE'csco_ht_development_test sys {
    //------------------------------------------------------------------------------
    // Define a Host Bridge model of what could be the BC12500 and provide the appropriate
    // parameters.
    //------------------------------------------------------------------------------

    // IO delay characteristics.
    keep BC12500.max_memory_latency == 10;
    keep BC12500.max_io_delay == 10;
    keep BC12500.max_tunneling_delay == 10;

    //------------------------------------------------------------------------------
    // Define the signal interface for the link.
    //------------------------------------------------------------------------------

    keep for each (interface) in BC12500.csco_ht_interfaces {

      //------------------------------------------------------------------------------
      // Define the error generation characteristics for the link interface
      //------------------------------------------------------------------------------
      interface.csco_ht_link.generate_bad_CRCs == FALSE;

      //------------------------------------------------------------------------------
      // Link TX scheduling and Knob Information
      //------------------------------------------------------------------------------
      // Set up weights that will influence the priority and selection frequency of packets to be transmitted in the tx_scheduling_queue
      // VC queue weights must be positive;
      interface.Posted_VC_queue_weight == 1;
      interface.NonPosted_VC_queue_weight == 1;
      interface.Response_VC_queue_weight == 1;

      // A weight to represent the priority of packets earlier in the tx_scheduling_queue to be selected for transmission.
      interface.scheduling_timing_weight == 1;

      // A weight to represent the priority of outstanding data packets to be selected for transmission.
      interface.data_scheduling_weight == 1;

      // These are a set of knobs that allow a user to determing the exact ordering requirements of the HT IO traffic.
      // These are generated from the ordering section 7.1 from the 1.0 HT spec.
      interface.Posted_Request_with_PassPW_pass_Posted_Request == TRUE;
      interface.NonPosted_Request_with_PassPW_pass_Posted_Request == TRUE;
      interface.NonPosted_Request_pass_NonPosted_Request == TRUE;
      interface.NonPosted_Request_pass_Response == TRUE;
      interface.Response_with_PassPW_pass_Posted_Request == TRUE;
      interface.Response_pass_Response == TRUE;

      // If this flag is set, the transmitter will release rx buffers in such a ways as to allow the other side of the link
      // to starve.  Once starvation has occured, the transmitter will wait a certain number of cycles before releasing buffers.
      interface.create_target_starvation == FALSE;
      interface.starvation_buffer_release_percentage == 10;

      // How long to wait for buffer releases once starvation has occurred
      interface.starvation_cycle_count == 0;

      // Nop's will be sent as a percentage of the BW.
      interface.nop_bandwidth_percent == 10;               // By default set Nop's to consume 10% of the bandwidth.

      // If this flag is set every nop packet will release the maximum available number of buffers.  Otherwise a random number
      // will be released with each packet.
      interface.nop_maximum_buffer_release == TRUE;

      // This field is the percent chance that a packet that is being transmitted will be interrupted.  If it is, a new packet will
      // be transmitted for this 4 byte window.
      interface.interrupt_tx_packet_percentage == 25;      // By default set the change interrupt to 25%.

      //------------------------------------------------------------------------------
      // Link TX Error Insertion
      //------------------------------------------------------------------------------
      // A global channel flag to turn on or off error injection on this TX link
      interface.allow_tx_errors == FALSE;

      // Set a percentage that will determine if a ctl packet will have a illegal ctl line
      interface.allow_illegal_ctl_packet in range [0..100];
    };
    //------------------------------------------------------------------------------
    // End definition of BC12500 Host Bridge Device.
    //------------------------------------------------------------------------------


    //------------------------------------------------------------------------------
    // Define a EOC Device that will sit on the chain after the Tunnel device and provide
    // the appropriate parameters.
    //------------------------------------------------------------------------------

    // IO delay characteristics.
    keep io_device_0.max_memory_latency == 10;
    keep io_device_0.max_io_delay == 10;
    keep io_device_0.max_tunneling_delay == 10;

    //------------------------------------------------------------------------------
    // Define the signal interface for the link.
    //------------------------------------------------------------------------------
    keep for each (interface) in io_device_0.csco_ht_interfaces {

      //------------------------------------------------------------------------------
      // Define the error generation characteristics for the link interface
      //------------------------------------------------------------------------------
      interface.csco_ht_link.generate_bad_CRCs == FALSE;

      //------------------------------------------------------------------------------
      // Link TX scheduling and Knob Information
      //------------------------------------------------------------------------------
      // Set up weights that will influence the priority and selection frequency of packets to be transmitted in the tx_scheduling_queue
      // VC queue weights must be positive;
      interface.Posted_VC_queue_weight == 1;
      interface.NonPosted_VC_queue_weight == 1;
      interface.Response_VC_queue_weight == 1;

      // A weight to represent the priority of packets earlier in the tx_scheduling_queue to be selected for transmission.
      interface.scheduling_timing_weight == 1;

      // A weight to represent the priority of outstanding data packets to be selected for transmission.
      interface.data_scheduling_weight == 1;

      // These are a set of knobs that allow a user to determing the exact ordering requirements of the HT IO traffic.
      // These are generated from the ordering section 7.1 from the 1.0 HT spec.
      interface.Posted_Request_with_PassPW_pass_Posted_Request == TRUE;
      interface.NonPosted_Request_with_PassPW_pass_Posted_Request == TRUE;
      interface.NonPosted_Request_pass_NonPosted_Request == TRUE;
      interface.NonPosted_Request_pass_Response == TRUE;
      interface.Response_with_PassPW_pass_Posted_Request == TRUE;
      interface.Response_pass_Response == TRUE;

      // If this flag is set, the transmitter will release rx buffers in such a ways as to allow the other side of the link
      // to starve.  Once starvation has occured, the transmitter will wait a certain number of cycles before releasing buffers.
      interface.create_target_starvation == FALSE;
      interface.starvation_buffer_release_percentage == 10;

      // How long to wait for buffer releases once starvation has occurred
      interface.starvation_cycle_count == 0;

      // Nop's will be sent as a percentage of the BW.
      interface.nop_bandwidth_percent == 10;               // By default set Nop's to consume 10% of the bandwidth.

      // If this flag is set every nop packet will release the maximum available number of buffers.  Otherwise a random number
      // will be released with each packet.
      interface.nop_maximum_buffer_release == TRUE;

      // This field is the percent chance that a packet that is being transmitted will be interrupted.  If it is, a new packet will
      // be transmitted for this 4 byte window.
      interface.interrupt_tx_packet_percentage == 25;      // By default set the change interrupt to 25%.

      //------------------------------------------------------------------------------
      // Link TX Error Insertion
      //------------------------------------------------------------------------------
      // A global channel flag to turn on or off error injection on this TX link
      interface.allow_tx_errors == FALSE;

      // Set a percentage that will determine if a ctl packet will have a illegal ctl line
      interface.allow_illegal_ctl_packet in range [0..100];
    };

    //------------------------------------------------------------------------------
    // End definition of io_device_0 EOC IO Device.
    //------------------------------------------------------------------------------
  };                                                       // when FALSE'csco_ht_development_test sys
};                                                         // End Extend Sys

//------------------------------------------------------------------------------
// Define Specman Traffic
//------------------------------------------------------------------------------

// Define new address regions for use in existing scenarios.
extend csco_ht_address_region : [test1, test2, test3, test4, test5];

extend test1'region csco_ht_address_map {                  // On the standard BAR config, this region belongs to the host bridge.
  keep base_addr == 40'h00_0000_0000;
  keep top_addr == 40'h00_0000_003F;
};

extend test2'region csco_ht_address_map {                  // On the standard BAR config, this region belongs to the host bridge.
  keep base_addr == 40'h00_2000_0000;
  keep top_addr == 40'h00_2000_003F;
};

extend test3'region csco_ht_address_map {                  // On the standard BAR config, this region belongs to the tunnel.
  keep base_addr == 40'h00_4000_0000;
  keep top_addr == 40'h00_4000_003F;
};

extend test4'region csco_ht_address_map {                  // On the standard BAR config, this region belongs to the eoc.
  keep base_addr == 40'h00_6000_0000;
  keep top_addr == 40'h00_6000_003F;
};

extend test5'region csco_ht_address_map {                  // On the standard BAR config, this region belongs to the eoc.
  keep base_addr == 40'h00_6000_0000;
  keep top_addr == 40'h00_6001_003F;
};

//----------------------------------------------------------------------------
// Base address for configuring the HT chain
//------------------------------------------------------------------------------

extend INIT_REGION'region csco_ht_address_map {
  keep base_addr == 40'h00_0000_0000;
};

//------------------------------------------------------------------------------
// PreConfigure Specman HT Devices
//------------------------------------------------------------------------------

// PreLoad the CSR's for each device.
extend csco_ht_device_control_block {
  pre_load_csr() is only {
  };
};

//------------------------------------------------------------------------------
// The target_memory_map is a structure used for a device when it is generating
// random traffic.  This map lets the device know what memory regions it can
// access.  This field is only defined for Tunnel and EOC devices as the Host
// will generate it's own target_memory_map based on chain sizing.
// Define the target address map for each device.
//------------------------------------------------------------------------------

extend csco_ht_device {
  // A flag used to indicate if the transmit and receive traffic should be recorded.
  keep display_records == FALSE;
  // A flag used to indicate that device level information should be printed.
  keep record_device_history == TRUE;
  // If this flag is set to false no data will be scoreboarded or checked.
  keep scoreboard.data_scoreboarding == TRUE;
};

extend Host_Bridge'device_function csco_ht_device {
  // A flag used to display information about the logical initialization of the ht chain.
  keep display_init_info == TRUE;
};

//------------------------------------------------------------------------------
// Debugging Flags
// Link Level Flags and Channel Level Flags can be customized on a per device basis,
// but that is considered to be an advanced feature as it is more difficult to do.
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// Link Level Flags
//------------------------------------------------------------------------------

extend csco_ht_link_interface {

  // Use these fields to control what information is printed to the screen.
  // A flag used to print link level information to the user interface.
  keep debug_link == FALSE;

  // A flag that will print CRC infomation to the user interface.
  keep debug_crc == FALSE;

  // A flag that will print initialization information to the user interface.
  keep debug_initialization == FALSE;

  // A flag that will cause the link interface to record every bit time that is transmitted and received.
  keep record_link_traffic == FALSE;
};

//--------------------------------------------------------------------------------
// Channel Level Flags
//--------------------------------------------------------------------------------

extend csco_ht_channel_interface {
  // If this flag is set a message will be printed whenever a non-Nop packet is transmitted or received.
  keep display_channel_messages == TRUE;

  // If this flag is set, then a message will be printed whenever a packet other than an empty Nop is transmitter or received.
  keep debug_channel == FALSE;

  // Monitor the level of the tx_scheduling_queue. If the watermark level is exceeded print a message.
  keep tx_scheduling_queue_watermark == 10;

  // These are flags that effect the reset sequencing.
  // Number of cycles to wait after RESET is deasserted before initiating the SYNC SEQUENCE.
  keep initiate_sync_sequence_delay in range [16..24];

  // Number of cycles for the sync sequence, minimun of CSCO_HT_MIN_SYNC_SEQUENCE_CYCLES
  keep sync_sequence_length == CSCO_HT_MIN_SYNC_SEQUENCE_CYCLES + 0;

};

//--------------------------------------------------------------------------------
// Device Level Flags
//--------------------------------------------------------------------------------

extend csco_ht_device_control_block {

  // If this flag is set, then traffic at the channel level will be recorded.
  keep record_traffic == TRUE;

  // When this flag is not set then any nop with no information in it is not recorded.  However, if that information
  // is necessary, enable this flag, and all channel level traffic will be retained.
  keep record_all_nop == FALSE;

};

//------------------------------------------------------------------------------
// For HT Test Bench Development Purposes Only (Remove for standard DUT work)
//------------------------------------------------------------------------------

extend sys {
  keep me.csco_ht_development_test == TRUE;

  when TRUE'csco_ht_development_test sys {
    host_bridge : csco_ht_device is instance;
    keep host_bridge.device_function == Host_Bridge;
    keep host_bridge.csco_ht_scene.size() == 1;
    keep for each in host_bridge.csco_ht_scene {
      .kind == READ_WRITE_IN_ORDER;
      .addr_map.region == test5;
      .addr_map.region == test5 => .num_transactions == 20;
    };

    keep for each in host_bridge.csco_ht_interfaces {
      it.csco_ht_link.tx_link == csco_ht_links[0];
      it.csco_ht_link.rx_link == csco_ht_links[1];

      //------------------------------------------------------------------------------
      // Link TX Error Insertion
      //------------------------------------------------------------------------------

      // A global channel flag to turn on or off error injection on this TX link
      it.allow_tx_errors == FALSE;

      // Set a percentage that will determine if a ctl packet will have a illegal ctl line
      it.allow_illegal_ctl_packet in range [0..100];
    };

    io_device_0 : csco_ht_device is instance;
    keep io_device_0.device_function == Tunnel_IO_Device;
    keep io_device_0.csco_ht_scene.size() == 0;

//    keep for each in io_device_0.csco_ht_scene {
//      .kind == READ_WRITE_IN_ORDER;
//      .addr_map.region == test1;
//    };

    keep for each in io_device_0.csco_ht_interfaces {
      index == 0 => it.csco_ht_link.tx_link == csco_ht_links[1];
      index == 0 => it.csco_ht_link.rx_link == csco_ht_links[0];
      index == 1 => it.csco_ht_link.tx_link == csco_ht_links[2];
      index == 1 => it.csco_ht_link.rx_link == csco_ht_links[3];

      //------------------------------------------------------------------------------
      // Link TX Error Insertion
      //------------------------------------------------------------------------------

      // A global channel flag to turn on or off error injection on this TX link
      it.allow_tx_errors == FALSE;

      // Set a percentage that will determine if a ctl packet will have a illegal ctl line
      it.allow_illegal_ctl_packet in range [0..100];
    };

    io_device_1 : csco_ht_device is instance;
    keep io_device_1.device_function == EOC_IO_Device;
    keep io_device_1.csco_ht_scene.size() == 0;
    keep for each in io_device_1.csco_ht_scene {
      .kind == INTERRUPT;
    };

    keep for each in io_device_1.csco_ht_interfaces {
      it.csco_ht_link.tx_link == csco_ht_links[3];
      it.csco_ht_link.rx_link == csco_ht_links[2];

      //------------------------------------------------------------------------------
      // Link TX Error Insertion
      //------------------------------------------------------------------------------

      // A global channel flag to turn on or off error injection on this TX link
      it.allow_tx_errors == FALSE;

      // Set a percentage that will determine if a ctl packet will have a illegal ctl line
      it.allow_illegal_ctl_packet in range [0..100];

    };

    csco_ht_links : list of csco_ht_link is instance;
    keep csco_ht_links.size() == 4;
  };
};

//--------------------------------------------------------------------------------
// Provide some general parameters to the global environment.
//--------------------------------------------------------------------------------

extend global {

  setup_test() is also {
    specman("break on error");
    // specman("collect events");
  };
};

'>

--------------------------------------------------------------------------------
-- Below are three examples of HT devices.  One Host Bridge, one Tunnel and one EOC device.
--------------------------------------------------------------------------------

  //------------------------------------------------------------------------------
  // Instantiate a basic Host Bridge device.
  //------------------------------------------------------------------------------
  example_host : csco_ht_device is instance;
    keep example_host.device_function == Host_Bridge;

  //------------------------------------------------------------------------------
  // Device Level Parameters
  //------------------------------------------------------------------------------
  // IO delay characteristics.
  keep example_host.max_memory_latency == 10;
  keep example_host.max_io_delay == 10;
  keep example_host.max_tunneling_delay == 10;

  //------------------------------------------------------------------------------
  // Define the signal interface for the link.
  //------------------------------------------------------------------------------

  keep for each (interface) in example_host.csco_ht_interfaces {

    //------------------------------------------------------------------------------
    // Define the error generation characteristics for the link interface
    //------------------------------------------------------------------------------
    interface.csco_ht_link.generate_bad_CRCs == FALSE;

    //------------------------------------------------------------------------------
    // Link TX scheduling and Knob Information
    //------------------------------------------------------------------------------

    // Set up weights that will influence the priority and selection frequency of packets to be transmitted in the tx_scheduling_queue
    // VC queue weights must be positive;
    interface.Posted_VC_queue_weight == 1;
    interface.NonPosted_VC_queue_weight == 1;
    interface.Response_VC_queue_weight == 1;

    // A weight to represent the priority of packets earlier in the tx_scheduling_queue to be selected for transmission.
    interface.scheduling_timing_weight == 1;

    // A weight to represent the priority of outstanding data packets to be selected for transmission.
    interface.data_scheduling_weight == 1;

    // These are a set of knobs that allow a user to determing the exact ordering requirements of the HT IO traffic.
    // These are generated from the ordering section 7.1 from the 1.0 HT spec.
    interface.Posted_Request_with_PassPW_pass_Posted_Request == TRUE;
    interface.NonPosted_Request_with_PassPW_pass_Posted_Request == TRUE;
    interface.NonPosted_Request_pass_NonPosted_Request == TRUE;
    interface.NonPosted_Request_pass_Response == TRUE;
    interface.Response_with_PassPW_pass_Posted_Request == TRUE;
    interface.Response_pass_Response == TRUE;

    // If this flag is set, the transmitter will release rx buffers in such a ways as to allow the other side of the link
    // to starve.  Once starvation has occured, the transmitter will wait a certain number of cycles before releasing buffers.
    interface.create_target_starvation == FALSE;
    interface.starvation_buffer_release_percentage == 10;

    // How long to wait for buffer releases once starvation has occurred
    interface.starvation_cycle_count == 0;

    // Nop's will be sent as a percentage of the BW.
    interface.nop_bandwidth_percent == 10;                 // By default set Nop's to consume 10% of the bandwidth.

    // If this flag is set every nop packet will release the maximum available number of buffers.  Otherwise a random number
    // will be released with each packet.
    interface.nop_maximum_buffer_release == TRUE;

    // This field is the percent chance that a packet that is being transmitted will be interrupted.  If it is, a new packet will
    // be transmitted for this 4 byte window.
    interface.interrupt_tx_packet_percentage == 25;        // By default set the change interrupt to 25%.

    //------------------------------------------------------------------------------
    // Link TX Error Insertion
    //------------------------------------------------------------------------------

    // A global channel flag to turn on or off error injection on this TX link
    interface.allow_tx_errors == FALSE;

    // Set a percentage that will determine if a ctl packet will have a illegal ctl line
    interface.allow_illegal_ctl_packet in range [0..100];
  };

  //------------------------------------------------------------------------------
  // End definition of basic Host Bridge Device.
  //------------------------------------------------------------------------------

  //------------------------------------------------------------------------------
  // Instantiate a basic Tunnel IO device.
  //------------------------------------------------------------------------------
  example_tunnel : csco_ht_device is instance;
  keep example_tunnel.device_function == Tunnel_IO_Device;

  //------------------------------------------------------------------------------
  // Device Level Parameters
  //------------------------------------------------------------------------------

  // IO delay characteristics.
  keep example_tunnel.max_memory_latency == 10;
  keep example_tunnel.max_io_delay == 10;
  keep example_tunnel.max_tunneling_delay == 10;

  //------------------------------------------------------------------------------
  // Define the signal interface for the link.
  // A tunnel has two interfaces so the definition of signals is more complicated.
  // a preface of "index == 0 =>" implies that the first interface should have the
  // specified value and a preface of "index == 1 =>" that the second interface
  // should have the specified value.
  //------------------------------------------------------------------------------

  keep for each (interface) using index (interface_index) in example_tunnel.csco_ht_interfaces {

    //------------------------------------------------------------------------------
    // Define the error generation characteristics for the link interface
    //------------------------------------------------------------------------------
    interface.csco_ht_link.generate_bad_CRCs == FALSE;

    //------------------------------------------------------------------------------
    // Link TX scheduling and Knob Information
    //------------------------------------------------------------------------------
    // Set up weights that will influence the priority and selection frequency of packets to be transmitted in the tx_scheduling_queue
    // VC queue weights must be positive;
    interface_index == 0 => interface.Posted_VC_queue_weight == 1;
    interface_index == 1 => interface.Posted_VC_queue_weight == 1;
    interface_index == 0 => interface.NonPosted_VC_queue_weight == 1;
    interface_index == 1 => interface.NonPosted_VC_queue_weight == 1;
    interface_index == 0 => interface.Response_VC_queue_weight == 1;
    interface_index == 1 => interface.Response_VC_queue_weight == 1;

    // A weight to represent the priority of packets earlier in the tx_scheduling_queue to be selected for transmission.
    interface_index == 0 => interface.scheduling_timing_weight == 1;
    interface_index == 1 => interface.scheduling_timing_weight == 1;

    // A weight to represent the priority of outstanding data packets to be selected for transmission.
    interface_index == 0 => interface.data_scheduling_weight == 1;
    interface_index == 1 => interface.data_scheduling_weight == 1;

    // These are a set of knobs that allow a user to determing the exact ordering requirements of the HT IO traffic.
    // These are generated from the ordering section 7.1 from the 1.0 HT spec.
    interface_index == 0 => interface.Posted_Request_with_PassPW_pass_Posted_Request == TRUE;
    interface_index == 1 => interface.Posted_Request_with_PassPW_pass_Posted_Request == TRUE;
    interface_index == 0 => interface.NonPosted_Request_with_PassPW_pass_Posted_Request == TRUE;
    interface_index == 1 => interface.NonPosted_Request_with_PassPW_pass_Posted_Request == TRUE;
    interface_index == 0 => interface.NonPosted_Request_pass_NonPosted_Request == TRUE;
    interface_index == 1 => interface.NonPosted_Request_pass_NonPosted_Request == TRUE;
    interface_index == 0 => interface.NonPosted_Request_pass_Response == TRUE;
    interface_index == 1 => interface.NonPosted_Request_pass_Response == TRUE;
    interface_index == 0 => interface.Response_with_PassPW_pass_Posted_Request == TRUE;
    interface_index == 1 => interface.Response_with_PassPW_pass_Posted_Request == TRUE;
    interface_index == 0 => interface.Response_pass_Response == TRUE;
    interface_index == 1 => interface.Response_pass_Response == TRUE;

    // If this flag is set, the transmitter will release rx buffers in such a ways as to allow the other side of the link
    // to starve.  Once starvation has occured, the transmitter will wait a certain number of cycles before releasing buffers.
    interface_index == 0 => interface.create_target_starvation == FALSE;
    interface_index == 1 => interface.create_target_starvation == FALSE;
    interface_index == 0 => interface.starvation_buffer_release_percentage == 10;
    interface_index == 1 => interface.starvation_buffer_release_percentage == 10;

    // How long to wait for buffer releases once starvation has occurred
    interface_index == 0 => interface.starvation_cycle_count == 0;
    interface_index == 1 => interface.starvation_cycle_count == 0;

    // Nop's will be sent as a percentage of the BW.
    interface_index == 0 => interface.nop_bandwidth_percent == 10; // By default set Nop's to consume 10% of the bandwidth.
    interface_index == 1 => interface.nop_bandwidth_percent == 10; // By default set Nop's to consume 10% of the bandwidth.

    // If this flag is set every nop packet will release the maximum available number of buffers.  Otherwise a random number
    // will be released with each packet.
    interface_index == 0 => interface.nop_maximum_buffer_release == TRUE;
    interface_index == 1 => interface.nop_maximum_buffer_release == TRUE;

    // This field is the percent chance that a packet that is being transmitted will be interrupted.  If it is, a new packet will
    // be transmitted for this 4 byte window.
    interface_index == 0 => interface.interrupt_tx_packet_percentage == 25;// By default set the change interrupt to 25%.
    interface_index == 1 => interface.interrupt_tx_packet_percentage == 25;// By default set the change interrupt to 25%.

    //------------------------------------------------------------------------------
    // Link TX Error Insertion
    //------------------------------------------------------------------------------
    // A global channel flag to turn on or off error injection on this TX link
    interface_index == 0 => interface.allow_tx_errors == FALSE;

    // Set a percentage that will determine if a ctl packet will have a illegal ctl line
    interface_index == 1 => interface.allow_illegal_ctl_packet in range [0..100];
  };

  //------------------------------------------------------------------------------
  // End definition of basic Tunnel IO Device.
  //------------------------------------------------------------------------------

  //------------------------------------------------------------------------------
  // Instantiate a basic EOC IO device.
  //------------------------------------------------------------------------------
  example_EOC : csco_ht_device is instance;
  keep example_EOC.device_function == EOC_IO_Device;

  //------------------------------------------------------------------------------
  // Device Level Parameters
  //------------------------------------------------------------------------------
  // IO delay characteristics.
  keep example_EOC.max_memory_latency == 10;
  keep example_EOC.max_io_delay == 10;
  keep example_EOC.max_tunneling_delay == 10;

  //------------------------------------------------------------------------------
  // Define the signal interface for the link.
  //------------------------------------------------------------------------------

  keep for each (interface) in example_EOC.csco_ht_interfaces {

    //------------------------------------------------------------------------------
    // Define the error generation characteristics for the link interface
    //------------------------------------------------------------------------------
    interface.csco_ht_link.generate_bad_CRCs == FALSE;

    //------------------------------------------------------------------------------
    // Link TX scheduling and Knob Information
    //------------------------------------------------------------------------------

    // Set up weights that will influence the priority and selection frequency of packets to be transmitted in the tx_scheduling_queue
    // VC queue weights must be positive;
    interface.Posted_VC_queue_weight == 1;
    interface.NonPosted_VC_queue_weight == 1;
    interface.Response_VC_queue_weight == 1;

    // A weight to represent the priority of packets earlier in the tx_scheduling_queue to be selected for transmission.
    interface.scheduling_timing_weight == 1;

    // A weight to represent the priority of outstanding data packets to be selected for transmission.
    interface.data_scheduling_weight == 1;

    // These are a set of knobs that allow a user to determing the exact ordering requirements of the HT IO traffic.
    // These are generated from the ordering section 7.1 from the 1.0 HT spec.
    interface.Posted_Request_with_PassPW_pass_Posted_Request == TRUE;
    interface.NonPosted_Request_with_PassPW_pass_Posted_Request == TRUE;
    interface.NonPosted_Request_pass_NonPosted_Request == TRUE;
    interface.NonPosted_Request_pass_Response == TRUE;
    interface.Response_with_PassPW_pass_Posted_Request == TRUE;
    interface.Response_pass_Response == TRUE;

    // If this flag is set, the transmitter will release rx buffers in such a ways as to allow the other side of the link
    // to starve.  Once starvation has occured, the transmitter will wait a certain number of cycles before releasing buffers.
    interface.create_target_starvation == FALSE;
    interface.starvation_buffer_release_percentage == 10;

    // How long to wait for buffer releases once starvation has occurred
    interface.starvation_cycle_count == 0;

    // Nop's will be sent as a percentage of the BW.
    interface.nop_bandwidth_percent == 10;                 // By default set Nop's to consume 10% of the bandwidth.

    // If this flag is set every nop packet will release the maximum available number of buffers.  Otherwise a random number
    // will be released with each packet.
    interface.nop_maximum_buffer_release == TRUE;

    // This field is the percent chance that a packet that is being transmitted will be interrupted.  If it is, a new packet will
    // be transmitted for this 4 byte window.
    interface.interrupt_tx_packet_percentage == 25;        // By default set the change interrupt to 25%.

    //------------------------------------------------------------------------------
    // Link TX Error Insertion
    //------------------------------------------------------------------------------
    // A global channel flag to turn on or off error injection on this TX link
    interface.allow_tx_errors == FALSE;

    // Set a percentage that will determine if a ctl packet will have a illegal ctl line
    interface.allow_illegal_ctl_packet in range [0..100];
  };

  //------------------------------------------------------------------------------
  // End definition of basic EOC IO Device.
  //------------------------------------------------------------------------------

//------------------------------------------------------------------------------------------
// To define directed traffic for a specman device a new scenario must be created.
// To do this, first define a name for the scenarion int he csco_ht_scenario_kind type.
// Then extend the csco_ht_scenario object to create a new when subtype that defines the
// generate_transactions method. After that is done extend the csco_ht_address_region type to
// create a target memory region and extend the csco_ht_address_map object to define the region.
// Use the schedule_packet() method to queue the request and use the directed methods
// to generate the request.  Below is an example for generating a sequence of 4 packets.
//------------------------------------------------------------------------------------------

extend csco_ht_address_region : [USER_REGION];

extend USER_REGION'region csco_ht_address_map {
  keep soft base_addr == 40'h00_1000_0000;
  keep top_addr == 40'h00_1100_0000;
};

extend csco_ht_scenario_kind : [USER_SCENE];

extend USER_SCENE'kind csco_ht_scenario {
  keep addr_map.region == USER_REGION;
  keep target_address_gen_complete == TRUE;                // Don't self generate target addresses

  generate_transactions()@parent_device.csco_ht_device_clock is {
    var delay : uint (bits:4);
    gen delay;
    wait [delay] * cycle;
    schedule_transaction(parent_device.gen_directed_write(DoubleWord_Write, NonPosted, NoPassPW, 4'b1000,  %{32'hCAFE}, 40'h1111100000));
    gen delay;
    wait [delay] * cycle;
    schedule_transaction(parent_device.gen_directed_write(Byte_Write, NonPosted,  NoPassPW, 4'b1001, %{32'hDEAD, 32'h0003}, 40'h2222200000));
    gen delay;
    wait [delay] * cycle;
    schedule_transaction(parent_device.gen_directed_read(DoubleWord_Read, NoPassPW, PassPW, 4'b1000, 4'b0000, 40'h1111100000));
    gen delay;
    wait [delay] * cycle;
    schedule_transaction(parent_device.gen_directed_read(Byte_Read, PassPW, NoPassPW, 4'b1001, 4'b1111, 40'h2222200000));
    done = TRUE;
  };

};

//------------------------------------------------------------------------------
// schedule_transaction() - This method will take a packet that is to be transmitted and place it in the channels scheduling queue.
// Additionally it will maintain the appropriate scoreboards and counters for the transmit path.  This method should be the
// only means of accessing the channel on the transmit side.
//------------------------------------------------------------------------------
  schedule_transaction(packet : csco_ht_packet);

//------------------------------------------------------------------------------
// Directed request packet generation
// These are a series of methods used for users to create directed HT requests.
// They allow a user to specify the major fields of the packet without worrying about the
// guts of an HT request.  Each method will return a newly created packet.
//------------------------------------------------------------------------------

  //------------------------------------------------------------------------------
  // gen_directed_flush()
  // passing_rule - Dictates if the PassPW bit will be set. NoPassPW or PassPW are the possible choices.
  // SeqID - Is the sequence number to be used for this request.
  //------------------------------------------------------------------------------

  gen_directed_flush(passing_rule : csco_ht_passing_rules, SeqID : uint (bits:4));

  //------------------------------------------------------------------------------
  // gen_directed_write()
  // data_type - Specifies if the request is to be a DoubleWord_Write or Byte_Write.  DoubleWord or Byte are the choices.
  // vc - Selects either the Posted or NonPosted virtual channels. Posted or NonPosted are the choices.
  // passing_rule - Dictates if the PassPW bit will be set. NoPassPW or PassPW are the possible choices.
  // SeqID - Is the sequence number to be used for this request.
  // data - Is the data to be written.
  // addr - The 40 bit address
  //------------------------------------------------------------------------------

  gen_directed_write(data_type : csco_ht_packet_encoding, vc : csco_ht_virtual_channel, passing_rule : csco_ht_passing_rules, SeqID : uint (bits:4), data : list of bit, addr : uint (bits:40));

  //------------------------------------------------------------------------------
  // gen_directed_read()
  // data_type - Specifies if the request is to be a DoubleWord_Read or Byte_Read.  DoubleWord or Byte are the choices.
  // resp_passing_rule - Will dictate if the PassPW bit will be set for the response. NoPassPW or PassPW are the possible choices.
  // passing_rule - Dictates if the PassPW bit will be set. NoPassPW or PassPW are the possible choices.
  // SeqID - Is the sequence number to be used for this request.
  // data_size - Is either the count field for DoubleWord request or the byte mask field for Byte requests.
  // addr - The 40 bit address
  //------------------------------------------------------------------------------

  gen_directed_read(data_type : csco_ht_packet_encoding, resp_passing_rule : csco_ht_passing_rules, passing_rule : csco_ht_passing_rules, SeqID : uint (bits:4), data_size : uint (bits:4), addr : uint (bits:40));

  //------------------------------------------------------------------------------
  // gen_directed_broadcast()
  // vc - Selects either the Posted or NonPosted virtual channels. Posted or NonPosted are the choices.
  // passing_rule - Dictates if the PassPW bit will be set. NoPassPW or PassPW are the possible choices.
  // SeqID - Is the sequence number to be used for this request.
  // addr - The 40 bit address
  //------------------------------------------------------------------------------

  gen_directed_broadcast(vc : csco_ht_virtual_channel, passing_rule : csco_ht_passing_rules, SeqID : uint (bits:4), addr : uint (bits:40));

  //------------------------------------------------------------------------------
  // gen_directed_fence()
  // passing_rule - Dictates if the PassPW bit will be set. NoPassPW or PassPW are the possible choices.
  // SeqID - Is the sequence number to be used for this request.
  //------------------------------------------------------------------------------

  gen_directed_fence(passing_rule : csco_ht_passing_rules, SeqID : uint (bits:4));

//------------------------------------------------------------------------------
// PreConfigure Specman HT Devices
// If you wanted to preconfigure a specman device the following examples could be used.
//------------------------------------------------------------------------------

// PreLoad the CSR's for each device.
extend csco_ht_device_control_block {
  pre_load_csr() is only {
    if(device_function == Host_Bridge) {
      // Base_Address_Register[0]
      device_header.write(6'h10, 4'hF, 32'h00000000);
      // Base_Address_Register[1]
      device_header.write(6'h14, 4'hF, 32'h00000000);
      // Base_Address_Register[2]
      device_header.write(6'h18, 4'hF, 32'h40000000);
      // Base_Address_Register[3]
      device_header.write(6'h1C, 4'hF, 32'h00000000);
      // Base_Address_Register[4]
      device_header.write(6'h20, 4'hF, 32'h80000000);
      // Base_Address_Register[5]
      device_header.write(6'h24, 4'hF, 32'h00000000);
    };

    if(device_function == Tunnel_IO_Device) {
      // Base_Address_Register[0]
      device_header.write(6'h10, 4'hF, 32'hC0000000);
      // Base_Address_Register[1]
      device_header.write(6'h14, 4'hF, 32'h00000000);
      // Base_Address_Register[2]
      device_header.write(6'h18, 4'hF, 32'h00000000);
      // Base_Address_Register[3]
      device_header.write(6'h1C, 4'hF, 32'h00000001);
      // Base_Address_Register[4]
      device_header.write(6'h20, 4'hF, 32'h40000000);
      // Base_Address_Register[5]
      device_header.write(6'h24, 4'hF, 32'h00000001);

      // Capability Command Register
      // set the UnitID to 1
      capability_register.write(6'h00, 4'hF, 32'h00010000);
    };

    if(device_function == EOC_IO_Device) {
      // Base_Address_Register[0]
      device_header.write(6'h10, 4'hF, 32'h80000000);
      // Base_Address_Register[1]
      device_header.write(6'h14, 4'hF, 32'h00000001);
      // Base_Address_Register[2]
      device_header.write(6'h18, 4'hF, 32'hC0000000);
      // Base_Address_Register[3]
      device_header.write(6'h1C, 4'hF, 32'h00000001);
      // Base_Address_Register[4]
      device_header.write(6'h20, 4'hF, 32'h00000000);
      // Base_Address_Register[5]
      device_header.write(6'h24, 4'hF, 32'h00000002);

      // Capability Command Register
      // set the UnitID to 2
      capability_register.write(6'h00, 4'hF, 32'h00020000);
    };
  };
};

